import re

from ansys.mapdl.core._commands import parse


def k(self, npt="", x="", y="", z="", **kwargs) -> int:
    """Define a keypoint.

    APDL Command: K

    Parameters
    ----------
    npt
        Reference number for keypoint.  If zero, the lowest
        available number is assigned [NUMSTR].

    x, y, z
        Keypoint location in the active coordinate system (may be
        R, θ, Z or R, θ, Φ).

    Returns
    -------
    int
        The keypoint number of the generated keypoint.

    Examples
    --------
    Create keypoint at ``(0, 1, 2)``

    >>> knum = mapdl.k('', 0, 1, 2)
    >>> knum
    1

    Create keypoint at ``(10, 11, 12)`` while specifying the
    keypoint number.

    >>> knum = mapdl.k(5, 10, 11, 12)
    >>> knum
    5

    Notes
    -----
    Defines a keypoint in the active coordinate system [CSYS] for
    line, area, and volume descriptions.  A previously defined
    keypoint of the same number will be redefined.  Keypoints may
    be redefined only if it is not yet attached to a line or is
    not yet meshed.  Solid modeling in a toroidal system is not
    recommended.
    """
    command = f"K,{npt},{x},{y},{z}"
    msg = self.run(command, **kwargs)

    if msg:
        if not re.search(r"KEYPOINT NUMBER", msg):
            res = re.search(r"(KEYPOINT\s*)([0-9]+)", msg)
        else:
            res = re.search(r"(KEYPOINT NUMBER =\s*)([0-9]+)", msg)

        if res:
            return int(res.group(2))


def kbetw(self, kp1="", kp2="", kpnew="", type_="", value="", **kwargs) -> int:
    """Creates a keypoint between two existing keypoints.

    APDL Command: KBETW

    Parameters
    ----------
    kp1
        First keypoint.

    kp2
        Second keypoint.

    kpnew
        Number assigned to the new keypoint.  Defaults to the
        lowest available keypoint number.

    type\_
        Type of input for VALUE.

        RATIO - Value is the ratio of the distances between keypoints as follows:
                ``(KP1-KPNEW)/(KP1-KP2)``.

        DIST - Value is the absolute distance between KP1 and
               KPNEW (valid only if current coordinate system is
               Cartesian).

    value
        Location of new keypoint, as defined by Type (defaults to
        0.5).  If VALUE is a ratio (Type = RATIO) and is less than
        0 or greater than 1, the keypoint is created on the
        extended line.  Similarly, if VALUE is a distance (Type =
        DIST) and is less than 0 or greater than the distance
        between KP1 and KP2, the keypoint is created on the
        extended line.

    Returns
    -------
    int
        Keypoint number of the generated keypoint.

    Examples
    --------
    Create a keypoint exactly centered between two keypoints.

    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 1, 0, 0)
    >>> k2 = mapdl.kbetw(k0, k1)
    >>> k2
    3

    Notes
    -----
    Placement of the new keypoint depends on the currently active
    coordinate system [CSYS].  If the coordinate system is
    Cartesian, the keypoint will lie on a straight line between
    KP1 and KP2.  If the system is not Cartesian (e.g.,
    cylindrical, spherical, etc.), the keypoint will be located as
    if on a line (which may not be straight) created in the
    current coordinate system between KP1 and KP2.  Note that
    solid modeling in a toroidal coordinate system is not
    recommended.
    """
    command = f"KBETW,{kp1},{kp2},{kpnew},{type_},{value}"
    return parse.parse_kpoint(self.run(command, **kwargs))


def kcenter(self, type_="", val1="", val2="", val3="", val4="", kpnew="",
            **kwargs) -> int:
    """Creates a keypoint at the center of a circular arc defined
    by three locations.

    APDL Command: KCENTER

    Parameters
    ----------
    type\_
        Type of entity used to define the circular arc.  The
        meaning of VAL1 through VAL4 will vary depending on Type.

        KP - Arc is defined by keypoints.

        LINE - Arc is defined by locations on a line.

    val1, val2, val3, val4
        Values used to specify three locations on the arc (see table
        below).

    kpnew
        Number assigned to new keypoint.  Defaults to the lowest available
        keypoint number.

    Returns
    -------
    int
        Keypoint number of the generated keypoint.

    Examples
    --------
    Create a keypoint at the center of a circle centered at (0, 0, 0)

    >>> k0 = mapdl.k("", 0, 1, 0)
    >>> k1 = mapdl.k("", 1, 0, 0)
    >>> k2 = mapdl.k("", 0, -1, 0)
    >>> k3 = mapdl.kcenter('KP', k0, k1, k2)
    >>> k3
    4

    Notes
    -----
    KCENTER should be used in the Cartesian coordinate system
    (CSYS,0) only.  This command provides three methods to define
    a keypoint at the center of three locations.

    Three keypoints:

             , - ~ ~ ~ - ,
         ,(x)(VAL2)        ' , <--- imaginary circluar arc
       ,                       ,
      ,                         ,
     ,                           ,
    (x)(VAL1)     (x)(KPNEW)    (x)(VAL3)

    """
    command = f"KCENTER,{type_},{val1},{val2},{val3},{val4},{kpnew}"
    return parse.parse_kpoint(self.run(command, **kwargs))


def kdele(self, np1="", np2="", ninc="", **kwargs):
    """Deletes unmeshed keypoints.

    APDL Command: KDELE

    Parameters
    ----------
    np1, np2, ninc
        Delete keypoints from NP1 to NP2 (defaults to NP1) in steps of NINC
        (defaults to 1).  If NP1 = ALL, NP2 and NINC are ignored and all
        selected keypoints [KSEL] are deleted.  If NP1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for NP1 (NP2 and NINC are ignored).

    Notes
    -----
    Deletes selected keypoints.  A keypoint attached to a line cannot be
    deleted unless the line is first deleted.
    """
    command = f"KDELE,{np1},{np2},{ninc}"
    return self.run(command, **kwargs)


def kdist(self, kp1="", kp2="", **kwargs) -> list:
    """Calculates and lists the distance between two keypoints.

    APDL Command: KDIST

    Parameters
    ----------
    kp1
        First keypoint in distance calculation.

    kp2
        Second keypoint in distance calculation.

    Returns
    -------
    list
        ``[X, Y, Z]`` distance between two keypoints.

    Examples
    --------
    Compute the distance between two keypoints.

    >>> kp0 = (0, 10, -3)
    >>> kp1 = (1, 5, 10)
    >>> knum0 = mapdl.k("", *kp0)
    >>> knum1 = mapdl.k("", *kp1)
    >>> dist = mapdl.kdist(knum0, knum1)
    >>> dist
    [1.0, -5.0, 13.0]

    Notes
    -----
    KDIST lists the distance between keypoints KP1 and KP2, as
    well as the current coordinate system offsets from KP1 to KP2,
    where the X, Y, and Z locations of KP1 are subtracted from the
    X, Y, and Z locations of KP2 (respectively) to determine the
    offsets.  KDIST is valid in any coordinate system except
    toroidal [CSYS,3].

    This command is valid in any processor.
    """
    return parse.parse_kdist(self.run(f"KDIST,{kp1},{kp2}", **kwargs))


def kfill(self, np1="", np2="", nfill="", nstrt="", ninc="", space="",
          **kwargs):
    """Generates keypoints between two keypoints.

    APDL Command: KFILL

    Parameters
    ----------
    np1, np2
        Beginning and ending keypoints for fill-in.  NP1 defaults to next
        to last keypoint specified, NP2 defaults to last keypoint
        specified.  If NP1 = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).

    nfill
        Fill NFILL keypoints between NP1 and NP2 (defaults to |NP2-NP1|-1).
        NFILL must be positive.

    nstrt
        Keypoint number assigned to first filled-in keypoint (defaults to
        NP1 + NINC).

    ninc
        Add this increment to each of the remaining filled-in keypoint
        numbers (may be positive or negative).  Defaults to
        (NP2-NP1)/(NFILL + 1), i.e., linear interpolation.

    space
        Spacing ratio.  Ratio of last division size to first division size.
        If > 1.0, divisions increase.  If < 1.0, divisions decrease.  Ratio
        defaults to 1.0 (uniform spacing).

    Notes
    -----
    Generates keypoints (in the active coordinate system) between two
    existing keypoints.  The two keypoints may have been defined in any
    coordinate system. However, solid modeling in a toroidal coordinate
    system is not recommended.   Any number of keypoints may be filled in
    and any keypoint numbering sequence may be assigned.
    """
    command = f"KFILL,{np1},{np2},{nfill},{nstrt},{ninc},{space}"
    return self.run(command, **kwargs)


def kgen(self, itime="", np1="", np2="", ninc="", dx="", dy="", dz="",
         kinc="", noelem="", imove="", **kwargs):
    """Generates additional keypoints from a pattern of keypoints.

    APDL Command: KGEN

    Parameters
    ----------
    itime
        Do this generation operation a total of ITIME times, incrementing
        all keypoints in the given pattern automatically (or by KINC) each
        time after the first.  ITIME must be more than 1 for generation to
        occur.

    np1, np2, ninc
        Generate keypoints from the pattern of keypoints beginning with NP1
        to NP2 (defaults to NP1) in steps of NINC (defaults to 1).  If NP1
        = ALL, NP2 and NINC are ignored and the pattern is all selected
        keypoints [KSEL].  If NP1 is negative, NP2 and NINC are ignored and
        the last |NP1| keypoints (in sequence from the highest keypoint
        number) are used as the pattern to be repeated.  If NP1 = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NP1 (NP2 and NINC are ignored).

    dx, dy, dz
        Keypoint location increments in the active coordinate system (DR,
        Dθ, DZ for cylindrical, DR, Dθ, DΦ for spherical).

    kinc
        Keypoint increment between generated sets.  If zero, the lowest
        available keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies if elements and nodes are also to be generated:

        0 - Generate nodes and point elements associated with the original keypoints, if
            they exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether keypoints will be moved or newly defined:

        0 - Generate additional keypoints as requested with the ITIME argument.

        1 - Move original keypoints to new position retaining the same keypoint numbers
            (ITIME, KINC, and NOELEM are ignored).  Valid only if the old
            keypoints are no longer needed at their original positions.
            Corresponding meshed items are also moved if not needed at
            their original position.

    Notes
    -----
    Generates additional keypoints (and corresponding mesh) from a given
    keypoint pattern.  The MAT, TYPE, REAL, and ESYS attributes are based
    upon the keypoints in the pattern and not upon the current settings.
    Generation is done in the active coordinate system.  Keypoints in the
    pattern may have been defined in any coordinate system.  However, solid
    modeling in a toroidal coordinate system is not recommended.
    """
    command = f"KGEN,{itime},{np1},{np2},{ninc},{dx},{dy},{dz},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def kl(self, nl1="", ratio="", nk1="", **kwargs) -> int:
    """Generates a keypoint at a specified location on an existing line.

    APDL Command: KL

    Parameters
    ----------
    nl1
        Number of the line.  If negative, the direction of line
        (as interpreted for RATIO) is reversed.

    ratio
        Ratio of line length to locate keypoint.  Must be between
        0.0 and 1.0.  Defaults to 0.5 (divide the line in half).

    nk1
        Number to be assigned to keypoint generated at division
        location (defaults to lowest available keypoint number
        [NUMSTR]).

    Returns
    -------
    int
        Keypoint number of the generated keypoint.

    Examples
    --------
    Create a keypoint on a line from (0, 0, 0) and (10, 0, 0)

    >>> kp0 = (0, 0, 0)
    >>> kp1 = (10, 0, 0)
    >>> knum0 = mapdl.k("", *kp0)
    >>> knum1 = mapdl.k("", *kp1)
    >>> lnum = mapdl.l(knum0, knum1)
    >>> lnum
    1

    """
    msg = self.run(f"KL,{nl1},{ratio},{nk1}", **kwargs)
    if msg:
        res = re.search(r'KEYPOINT\s+(\d+)\s+', msg)
        if res is not None:
            return int(res.group(1))


def klist(self, np1="", np2="", ninc="", lab="", **kwargs):
    """Lists the defined keypoints or hard points.

    APDL Command: KLIST

    Parameters
    ----------
    np1, np2, ninc
        List keypoints from NP1 to NP2 (defaults to NP1) in steps of NINC
        (defaults to 1).  If NP1 = ALL (default), NP2 and NINC are ignored
        and all selected keypoints [KSEL] are listed.  If NP1 = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NP1 (NP2 and NINC are ignored).

    lab
        Coordinate listing key:

        (blank) - List all keypoint information.

        COORD - Suppress all but the keypoint coordinates (shown to a higher degree of accuracy
                than when displayed with all information).

        HPT - List only hard point information.

    Notes
    -----
    Lists keypoints in the active display coordinate system [DSYS].  An
    attribute (TYPE, MAT, REAL, or ESYS) listed as a zero is unassigned;
    one listed as a positive value indicates that the attribute was
    assigned with the KATT command (and will not be reset to zero if the
    mesh is cleared); one listed as a negative value indicates that the
    attribute was assigned using the attribute pointer [TYPE, MAT, REAL, or
    ESYS] that was active during meshing (and will be reset to zero if the
    mesh is cleared).

    This command is valid in any processor.
    """
    command = f"KLIST,{np1},{np2},{ninc},{lab}"
    return self.run(command, **kwargs)


def kmodif(self, npt="", x="", y="", z="", **kwargs):
    """Modifies an existing keypoint.

    APDL Command: KMODIF

    Parameters
    ----------
    npt
        Modify coordinates of this keypoint.  If NPT = ALL, modify
        coordinates of all selected keypoints [KSEL].  If NPT = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NPT.

    x, y, z
        Replace the previous coordinate values assigned to this keypoint
        with these corresponding coordinate values.  Values are interpreted
        according to the active coordinate system (R, θ, Z for cylindrical,
        R, θ,Φ for spherical).  If X = P, graphical picking is used to
        locate keypoint and Y and Z are ignored.  A blank retains the
        previous value.  You cannot specify Y = P.

    Notes
    -----
    Lines, areas, and volumes attached to the modified keypoint (if any)
    must all be selected and will be redefined using the active coordinate
    system.  However, solid modeling in a toroidal coordinate system is not
    recommended.

    Caution:: : Redefined entities may be removed from any defined
    components and assemblies. Nodes and elements will be automatically
    cleared from any redefined keypoints, lines, areas, or volumes.

    The KMODIF command moves keypoints for geometry modification without
    validating underlying entities. To merge keypoints and update higher
    order entities, issue the NUMMRG command instead.
    """
    command = f"KMODIF,{npt},{x},{y},{z}"
    return self.run(command, **kwargs)


def kmove(self, npt="", kc1="", x1="", y1="", z1="", kc2="", x2="", y2="",
          z2="", **kwargs):
    """Calculates and moves a keypoint to an intersection.

    APDL Command: KMOVE

    Parameters
    ----------
    npt
        Move this keypoint.  If NPT = P, graphical picking is enabled and
        all remaining command fields are ignored (valid only in the GUI).
        A component name may also be substituted for NPT.

    kc1
        First coordinate system number.  Defaults to 0 (global Cartesian).

    x1, y1, z1
        Input one or two values defining the location of the keypoint in
        this coordinate system.  Input "U" for unknown value(s) to be
        calculated and input "E" to use an existing coordinate value.
        Fields are R1, θ1, Z1 for cylindrical, or R1, θ1, ϕ1 for spherical.

    kc2
        Second coordinate system number.

    x2, y2, z2
        Input two or one value(s) defining the location of the keypoint in
        this coordinate system.  Input "U" for unknown value(s) to be
        calculated and input "E" to use an existing coordinate value.
        Arguments are R2, θ2, Z2 for cylindrical, or R2, θ2, ϕ2 for
        spherical.

    Notes
    -----
    Calculates and moves a keypoint to an intersection location.  The
    keypoint must have been previously defined (at an approximate location)
    or left undefined (in which case it is internally defined at the SOURCE
    location).  The actual location is calculated from the intersection of
    three surfaces (implied from three coordinate constants in two
    different coordinate systems).  Note that solid modeling in a toroidal
    coordinate system is not recommended.  See the MOVE command for surface
    and intersection details.  The three (of six) constants easiest to
    define should be used.  The program will calculate the remaining three
    coordinate constants.  All arguments, except KC1, must be input.  Use
    the repeat command [*REPEAT] after the KMOVE command to move a series
    of keypoints, if desired.
    """
    command = f"KMOVE,{npt},{kc1},{x1},{y1},{z1},{kc2},{x2},{y2},{z2}"
    return self.run(command, **kwargs)


def knode(self, npt="", node="", **kwargs) -> int:
    """Defines a keypoint at an existing node location.

    APDL Command: KNODE

    Parameters
    ----------
    npt
        Arbitrary reference number for keypoint.  If zero, the
        lowest available number is assigned [NUMSTR].

    node
        Node number defining global X, Y, Z keypoint location.  A
        component name may also be substituted for NODE.

    Returns
    -------
    int
        Keypoint number of the generated keypoint.

    Examples
    --------
    Create a keypoint at a node at (1, 2, 3)

    >>> nnum = mapdl.n('', 1, 2, 3)
    >>> knum1 = mapdl.knode('', nnum)
    >>> knum1
    1

    """
    msg = self.run(f"KNODE,{npt},{node}", **kwargs)
    if msg:
        res = re.search(r'KEYPOINT NUMBER =\s+(\d+)', msg)
        if res is not None:
            return int(res.group(1))


def kplot(self, np1="", np2="", ninc="", lab="", **kwargs):
    """Displays the selected keypoints.

    APDL Command: KPLOT

    Parameters
    ----------
    np1, np2, ninc
        Display keypoints from NP1 to NP2 (defaults to NP1) in steps of
        NINC (defaults to 1).  If NP1 = ALL (default), NP2 and NINC are
        ignored and all selected keypoints [KSEL] are displayed.

    lab
        Determines what keypoints are plotted (one of the following):

        (blank) - Plots all keypoints.

        HPT - Plots only those keypoints that are hard points.

    Notes
    -----
    This command is valid in any processor.
    """
    command = f"KPLOT,{np1},{np2},{ninc},{lab}"
    return self.run(command, **kwargs)


def kpscale(self, np1="", np2="", ninc="", rx="", ry="", rz="", kinc="",
            noelem="", imove="", **kwargs):
    """Generates a scaled set of (meshed) keypoints from a pattern of

    APDL Command: KPSCALE
    keypoints.

    Parameters
    ----------
    np1, np2, ninc
        Set of keypoints (NP1 to NP2 in steps of NINC) that defines the
        pattern to be scaled.  NP2 defaults to NP1, NINC defaults to 1.  If
        NP1 = ALL, NP2 and NINC are ignored and the pattern is defined by
        all selected keypoints.  If NP1 = P, graphical picking is enabled
        and all remaining command fields are ignored (valid only in the
        GUI).  A component name may also be substituted for NP1 (NP2 and
        NINC are ignored).

    rx, ry, rz
        Scale factors to be applied to the X, Y, Z keypoint coordinates in
        the active coordinate system (RR, Rθ, RZ for cylindrical; RR, Rθ,
        RΦ for spherical).  The Rθ and RΦ scale factors are interpreted as
        angular offsets.  For example, if CSYS = 1, an RX, RY, RZ input of
        (1.5,10,3) would scale the specified keypoints 1.5 times in the
        radial and 3 times in the Z direction, while adding an offset of 10
        degrees to the keypoints.)  Zero, blank, or negative scale factor
        values are assumed to be 1.0.  Zero or blank angular offsets have
        no effect.

    kinc
        Increment to be applied to the keypoint numbers for generated set.
        If zero, the lowest available keypoint numbers will be assigned
        [NUMSTR].

    noelem
        Specifies whether nodes and elements are also to be generated:

        0 - Nodes and point elements associated with the original keypoints will be
            generated (scaled) if they exist.

        1 - Nodes and point elements will not be generated.

    imove
        Specifies whether keypoints will be moved or newly defined:

        0 - Additional keypoints will be generated.

        1 - Original keypoints will be moved to new position (KINC and NOELEM are ignored).
            Use only if the old keypoints are no longer needed at their
            original positions.  Corresponding meshed items are also moved
            if not needed at their original position.

    Notes
    -----
    Generates a scaled set of keypoints (and corresponding mesh) from a
    pattern of keypoints.  The MAT, TYPE, REAL, and ESYS attributes are
    based on the keypoints in the pattern and not the current settings.
    Scaling is done in the active coordinate system.  Keypoints in the
    pattern could have been generated in any coordinate system.  However,
    solid modeling in a toroidal coordinate system is not recommended.
    """
    command = f"KPSCALE,{np1},{np2},{ninc},{rx},{ry},{rz},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def kscale(self, kinc="", np1="", np2="", ninc="", rx="", ry="", rz="",
           **kwargs):
    """Generates a scaled pattern of keypoints from a given keypoint pattern.

    APDL Command: KSCALE

    Parameters
    ----------
    kinc
        Do this scaling operation one time, incrementing all keypoints in
        the given pattern by KINC.  If KINC = 0, keypoints will be
        redefined at the scaled locations.

    np1, np2, ninc
        Scale keypoints from pattern beginning with NP1 to NP2 (defaults to
        NP1) in steps of NINC (defaults to 1).  If NP1 = ALL, NP2 and NINC
        are ignored and pattern is all selected keypoints [KSEL].  If NP1 =
        P, graphical picking is enabled and all remaining command fields
        are ignored (valid only in the GUI).  A component name may also be
        substituted for NP1 (NP2 and NINC are ignored).

    rx, ry, rz
        Scale factor ratios.  Scaling is relative to the origin of the
        active coordinate system (RR, Rθ, RZ for cylindrical, RR, Rθ, RΦ
        for spherical).  If > 1.0, pattern is enlarged.  If < 1.0, pattern
        is reduced.  Ratios each default to 1.0.

    Notes
    -----
    Generates a scaled pattern of keypoints from a given keypoint pattern.
    Scaling is done in the active coordinate system (see analogous node
    scaling [NSCALE]).  Solid modeling in a toroidal coordinate system is
    not recommended.
    """
    command = f"KSCALE,{kinc},{np1},{np2},{ninc},{rx},{ry},{rz}"
    return self.run(command, **kwargs)


def ksum(self, **kwargs):
    """Calculates and prints geometry statistics of the selected keypoints.

    APDL Command: KSUM

    Notes
    -----
    Calculates and prints geometry statistics (centroid location, moments
    of inertia, etc.) associated with the selected keypoints.  Geometry
    items are reported in the global Cartesian coordinate system.  A unit
    density is assumed, irrespective of any material associations [KATT,
    MAT].  Items calculated by KSUM and later retrieved by a *GET or *VGET
    command are valid only if the model is not modified after the KSUM
    command is issued.
    """
    command = f"KSUM,"
    return self.run(command, **kwargs)


def ksymm(self, ncomp="", np1="", np2="", ninc="", kinc="", noelem="",
          imove="", **kwargs):
    """Generates a reflected set of keypoints.

    APDL Command: KSYMM

    Parameters
    ----------
    ncomp
        Symmetry key:

        X - X (or R) symmetry (default).

        Y - Y (or θ) symmetry.

        Z - Z (or Φ) symmetry.

    np1, np2, ninc
        Reflect keypoints from pattern beginning with NP1 to NP2 (defaults
        to NP1) in steps of NINC (defaults to 1).  If NP1 = ALL, NP2 and
        NINC are ignored and pattern is all selected keypoints [KSEL].  If
        Ncomp = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NP1 (NP2 and NINC are ignored).

    kinc
        Keypoint increment between sets.  If zero, the lowest available
        keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies whether nodes and elements are also to be generated:

        0 - Generate nodes and point elements associated with the original keypoints, if
            they exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether keypoints will be moved or newly defined:

        0 - Generate additional keypoints.

        1 - Move original keypoints to new position retaining the same keypoint numbers
            (KINC and NOELEM are ignored).  Valid only if the old keypoints
            are no longer needed at their original positions.
            Corresponding meshed items are also moved if not needed at
            their original position.

    Notes
    -----
    Generates a reflected set of keypoints (and corresponding mesh) from a
    given keypoint pattern by a symmetry reflection (see analogous node
    symmetry command, NSYM).  The MAT, TYPE, REAL, and ESYS attributes are
    based upon the keypoints in the pattern and not upon the current
    settings.  Reflection is done in the active coordinate system by
    changing a particular coordinate sign.  Keypoints in the pattern may
    have been generated in any coordinate system.  However, solid modeling
    in a toroidal coordinate system is not recommended.
    """
    command = f"KSYMM,{ncomp},{np1},{np2},{ninc},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def ktran(self, kcnto="", np1="", np2="", ninc="", kinc="", noelem="",
          imove="", **kwargs):
    """Transfers a pattern of keypoints to another coordinate system.

    APDL Command: KTRAN

    Parameters
    ----------
    kcnto
        Reference number of coordinate system where the pattern is to be
        transferred.  Transfer occurs from the active coordinate system.

    np1, np2, ninc
        Transfer keypoints from pattern beginning with NP1 to NP2 (defaults
        to NP1) in steps of NINC (defaults to 1).  If NP1 = ALL, NP2 and
        NINC are ignored and pattern is all selected keypoints [KSEL].  If
        NP1 = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NP1 (NP2 and NINC are ignored).

    kinc
        Keypoint increment between sets.  If zero, the lowest available
        keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies whether nodes and elements are also to be generated:

        0 - Generate nodes and point elements associated with the original keypoints, if
            they exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether keypoints will be moved or newly defined:

        0 - Generate additional keypoints.

        1 - Move original keypoints to new position retaining the same keypoint numbers
            (KINC and NOELEM are ignored).  Valid only if the old keypoints
            are no longer needed at their original positions.
            Corresponding meshed items are also moved if not needed at
            their original position.

    Notes
    -----
    Transfers a pattern of keypoints (and corresponding mesh) from one
    coordinate system to another (see analogous node transfer command,
    TRANSFER).  The MAT, TYPE, REAL, and ESYS attributes are based upon the
    keypoints in the pattern and not upon the current settings.  Coordinate
    systems may be translated and rotated relative to each other.  Initial
    pattern may be generated in any coordinate system.  Coordinate values
    are interpreted in the active coordinate system and are transferred
    directly.  Solid modeling in a toroidal coordinate system is not
    recommended.
    """
    command = f"KTRAN,{kcnto},{np1},{np2},{ninc},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def source(self, x="", y="", z="", **kwargs):
    """Defines a default location for undefined nodes or keypoints.

    APDL Command: SOURCE

    Parameters
    ----------
    x, y, z
        Global Cartesian coordinates for source nodes or keypoints
        (defaults to the origin).

    Notes
    -----
    Defines a global Cartesian location for undefined nodes or keypoints
    moved during intersection calculations [MOVE or KMOVE].
    """
    command = f"SOURCE,{x},{y},{z}"
    return self.run(command, **kwargs)



def hemiopt(self, hres="", **kwargs):
    """Specifies options for Hemicube view factor calculation.

    APDL Command: HEMIOPT

    Parameters
    ----------
    hres
        Hemicube resolution.  Increase value to increase the accuracy of
        the view factor calculation.  Defaults to 10.
    """
    command = f"HEMIOPT,{hres}"
    return self.run(command, **kwargs)


def radopt(self, fluxtol="", solver="", maxiter="",
           toler="", overrlex="", maxfluxiter="", **kwargs):
    """Specifies Radiosity Solver options.

    APDL Command: RADOPT

    Parameters
    ----------
    fluxtol
        Convergence tolerance for radiation flux. Defaults to 0.0001. This
        value is a relative tolerance.

    solver
        Choice of solver for radiosity calculation:

        0 - Gauss-Seidel iterative solver (default).

        1 - Direct solver.

        2 - Jacobi solver.

    maxiter
        Maximum number of iterations for iterative solver (SOLVER = 0 or
        2). Defaults to 1000.

    toler
        Convergence tolerance for the iterative solver (SOLVER = 0 or 2).
        Defaults to 0.1.

    overrlex
        Over-relaxation factor applied to the iterative solver (SOLVER = 0
        or 2). Defaults to 0.1.

    maxfluxiter
        Maximum number of flux iterations to be performed according to the
        specified solver type:

        0 - If the FULL solver is specified (THOPT,FULL), convergence criteria are
            monitored and iterations are performed until convergence
            occurs. If the QUASI solver is specified (THOPT,QUASI),
            convergence criteria are ignored and one iteration is
            performed. This value is the default.

        1, 2, 3, ...N  - If the FULL solver is specified
                         (THOPT,FULL), convergence criteria are
                         monitored and iterations are performed
                         until convergence occurs, or until the
                         specified number of iterations has been
                         completed, whichever comes first. If the
                         QUASI solver is specified (THOPT,QUASI),
                         convergence criteria are ignored and the
                         specified number of iterations are
                         completed.

    Notes
    -----
    The radiation heat flux is linearized, resulting in robust convergence.

    The radiation flux norm for FLUXTOL is expressed as:

    where i is the pass or iteration number and j is the surface facet for
    radiation.

    For a sufficiently small absolute tolerance value, relative tolerance
    converges in fewer iterations than absolute tolerance. For a
    sufficiently large absolute tolerance value, relative tolerance may
    cause convergence difficulties.

    For more information about FLUXTOL and MAXFLUXITER usage, see Figure:
    3.5:: FULL Solution Method When Radiosity Is Present and Figure: 3.6::
    QUASI Solution Method When Radiosity Is Present in the Thermal Analysis
    Guide.

    In Figure: 3.5:: FULL Solution Method When Radiosity Is Present and
    Figure: 3.6:: QUASI Solution Method When Radiosity Is Present (under
    Solving for Temperature and Radiosity in the Thermal Analysis Guide),
    refer to the KQQ = FQ equation system via the iterative method:

    If TOLER ≥ 0, the iterative solver is converged for maximum value over
    a different j as shown:

    If TOLER < 0, the iterative solver is converged for maximum value over
    a different j as shown:

    where:

    The Jacobi solver (SOLVER = 2) is suitable when using Distributed
    ANSYS. This option is only available for 3-D models; if SOLVER is set
    to 2 for a 2-D analysis, the Gauss-Seidel iterative solver (SOLVER = 0)
    is used.
    """
    command = f"RADOPT,,{fluxtol},{solver},{maxiter},{toler},{overrlex},,,,,{maxfluxiter}"
    return self.run(command, **kwargs)


def spcnod(self, encl="", node="", **kwargs):
    """Defines a space node for radiation using the Radiosity method.

    APDL Command: SPCNOD

    Parameters
    ----------
    encl
        Radiating surface enclosure number.  Defaults to 1. If ENCL = STAT,
        the command lists all enclosure space nodes. If  ENCL =  DELE, the
        command deletes all enclosure space nodes.

    node
        Node defined to be the space node.

    Notes
    -----
    For open systems, an enclosure may radiate to a space node (NODE).

    Open systems may be characterized by one or more enclosures (ENCL).
    Each enclosure may radiate to a different space node (NODE).

    For a space node that is not part of the finite element model, specify
    the temperature using the D command. For the first load step, the space
    node temperature ramps from the uniform temperature specified by the
    TUNIF command to the temperature specified by the D command. For
    subsequent load steps, it ramps from the previous value of the space
    node temperature. For intermediate load steps, use the SPCNOD,DELETE
    command and specify the space node temperature again to ramp from the
    uniform temperature.

    For a space node that is part of the finite element model, the
    temperature is that calculated during the finite element solution.
    """
    command = f"SPCNOD,{encl},{node}"
    return self.run(command, **kwargs)


def spctemp(self, encl="", temp="", **kwargs):
    """Defines a free-space ambient temperature for radiation using the

    APDL Command: SPCTEMP
    Radiosity method.

    Parameters
    ----------
    encl
        Radiating surface enclosure number. Defaults to 1. If ENCL = STAT,
        the command lists all enclosure space temperatures.  If ENCL =
        DELE, the command deletes all enclosure space temperatures.

    temp
        Temperature of free-space in the reference temperature system.  The
        temperature will be offset by the value specified in the TOFFST
        command for internal calculations.

    Notes
    -----
    For open systems, an enclosure may radiate to the free-space ambient
    temperature (TEMP).

    Open systems may be characterized by one or more enclosures (ENCL).
    Each enclosure may radiate to a different free-space ambient
    temperature  (TEMP).

    For the first load step, the space temperature ramps from the uniform
    temperature specified by the TUNIF command to the temperature specified
    by the SPCTEMP command. For subsequent load steps, it ramps from the
    previous value of the space temperature. For intermediate load steps,
    use the SPCTEMP,DELETE command and specify the space temperature again
    to ramp from the uniform temperature.

    If using SPCTEMP with the ANSYS Multi-field solver (MFS),  you must
    capture this command in the command file using MFCMMAND. This step is
    necessary because at the end of each field computation, this command is
    unset.
    """
    command = f"SPCTEMP,{encl},{temp}"
    return self.run(command, **kwargs)


def v2dopt(self, geom="", ndiv="", hidopt="", nzone="", **kwargs):
    """Specifies 2-D/axisymmetric view factor calculation options.

    APDL Command: V2DOPT

    Parameters
    ----------
    geom
        Choice of geometry:

        0 - Planar (default).

        1 - Axisymmetric

    ndiv
         Number of divisions for axisymmetric geometry (that is, the number
        of circumferential segments). Default is 20. Maximum is 90.

    hidopt
        Viewing option:

        0 - Hidden (default).

        1 - Non-hidden

    nzone
        Number of zones (that is, the number of rays emanating from a
        surface) for view factor calculation. Default is 200. Maximum is
        1000.
    """
    command = f"V2DOPT,{geom},{ndiv},{hidopt},{nzone}"
    return self.run(command, **kwargs)


def vfsm(self, action="", encl="", opt="", maxiter="", conv="", **kwargs):
        """Adjusts view factor matrix to satisfy reciprocity and/or
        row sum properties.

        APDL Command: VFSM

        Parameters
        ----------
        action
            Action to be performed:

            Define - Define a view factor summation (default)

            Clear - Resets the scaling method to 0 for all
                    enclosures. All subsequent arguments are ignored.

            Status - Outputs the OPT value for each enclosure in the model.

        encl
            Previously defined enclosure number for the view factor adjustment.

        opt
            Option key:

            0 - The view factor matrix values are not adjusted (default).

            1 - The view factor matrix values are adjusted so that the
                row sum equals 1.0.

            2 - The view factor matrix values are adjusted so that the
                row sum equals 1.0 and the reciprocity relationship is
                satisfied.

            3 - The view factor matrix values are adjusted so that the
                original row sum is maintained.

            4 - The view factor matrix values are adjusted so that the
                original row sum is maintained and the reciprocity
                relationship is satisfied.

        maxiter
            Maximum number of iterations to achieve convergence. Valid only
            when OPT = 2 or 4. Default is 100.

        conv
            Convergence value for row sum. Iterations will continue (up to
            MAXITER) until the maximum residual over all the rows is less than
            this value. Valid only when OPT = 2 or 4. Default is 1E-3.

        Notes
        -----
        To have a good energy balance, it is important to satisfy both the row
        sum and reciprocity relationships. For more information, see View
        Factors in the Mechanical APDL Theory Reference.

        OPT = 1 and 2 are suitable for perfect enclosures. OPT = 1 is less
        expensive than OPT = 2 because no iterations are involved. However,
        with OPT = 1, the reciprocity relationship is not satisfied.

        OPT = 3 and 4 are suitable for leaky enclosures. OPT = 3 is less
        expensive than OPT = 4 because no iterations are involved. However,
        with OPT = 3, the reciprocity relationship is not satisfied.

        The VFSM command must be used before VFOPT is issued, or Solve is
        initiated.
        """
        command = f"VFSM,{action},{encl},{opt},{maxiter},{conv}"
        return self.run(command, **kwargs)


def vfopt(self, opt="", filename="", ext="", dir_="", filetype="",
          fileformat="", **kwargs):
    """Specifies options for the view factor file and calculates view factors.

    APDL Command: VFOPT

    Parameters
    ----------
    opt
        View factor option:

        NEW - Calculate view factors and write them to a file.

        OFF - Do not recalculate view factors it they already exist in the database,
              otherwise calculate compute them. This option is the default
              behavior.

        READ - Read view factors from a binary file. For subsequent SOLVE commands, switch to
               the default option (OFF).

        NONE - Do not write view factors to a file.

    fname
        File name for view factor matrix. Default = Jobname.

    ext
        Filename extension for view factor matrix. Default = .vf.

    dir\_
        Directory path for view factor matrix. If you do not specify a
        directory path, it will default to your working directory.

    filetype
        View factor file type:

        BINA - Binary (default).

        ASCI - ASCII.

    fileformat
        Format for the specified Filetype:

        Binary files (Filetype = BINA): - 0

        No compression. (View factor file size may be very large.) - 1

    Notes
    -----
    The VFOPT command allows you to deactivate the view factor computation
    (Opt = OFF) if the view factors already exist in the database. The
    default behavior is OFF upon encountering the second and subsequent
    SOLVE commands in the solution processor.

    When Opt = READ, only a previously calculated view factor binary file
    is valid.

    For 3-D analyses, two options are available for calculating view
    factors when running Distributed ANSYS:

    Issue a SOLVE command -- View factors are calculated in parallel mode
    if no view factors were previously calculated.

    Issue a VFOPT,NEW command -- View factors are calculated in serial
    mode.

    For 2-D analyses, view factors are calculated in serial mode.
    """
    command = f"VFOPT,{opt},{filename},{ext},{dir_},{filetype},{fileformat}"
    return self.run(command, **kwargs)


def vfquery(self, srcelem="", tarelem="", **kwargs):
    """Queries and prints element Hemicube view factors and average view

    APDL Command: VFQUERY
    factor.

    Parameters
    ----------
    srcelem
        Elements representing the source radiating surfaces used to query
        the view factor at the target element(s). If SRCELEM = P, graphical
        picking is enabled (valid only in the GUI). If SRCELEM = ALL, all
        selected elements will have their view factors queried. A component
        name may also be substituted for SRCELEM. Selected elements must be
        flagged for surface to surface radiation in order to query view
        factors (SF, SFA, or SFE with Lab = RDSF). The view factors must
        have been previously computed.

    tarelem
        Element for view factor query. If TARELEM = P, graphical picking is
        enabled (valid only in the GUI). If TARELEM = ALL, all selected
        elements will have their view factors queried. A component name may
        also be substituted for TARELEM. Selected elements must be flagged
        for surface to surface radiation in order to query view factors
        (SF, SFA, or SFE with Lab = RDSF). The view factors must have been
        previously computed.

    Notes
    -----
    View factors for each target element will be printed.

    An average view factor for all target elements will be computed.  (Use
    *GET  to retrieve the average value).

    When resuming a database, issue the command VFOPT,READ before issuing
    the VFQUERY command.
    """
    command = f"VFQUERY,{srcelem},{tarelem}"
    return self.run(command, **kwargs)



from ansys.mapdl.core._commands import parse


def aadd(self, na1="", na2="", na3="", na4="", na5="", na6="", na7="",
         na8="", na9="", **kwargs):
    """Adds separate areas to create a single area.

    APDL Command: AADD

    Parameters
    ----------
    na1, na2, na3, . . . , na9
        Numbers of areas to be added.  If NA1 = ALL, add all selected areas
        and ignore NA2 to NA9.  If NA1 = P, graphical picking is enabled
        and all remaining arguments are ignored (valid only in the GUI).  A
        component name may also be substituted for NA1.

    Notes
    -----
    The areas must be coplanar.  The original areas (and their
    corresponding lines and keypoints) will be deleted by default.
    See the BOPTN command for the options available to Boolean
    operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be
    transferred to the new entities generated.  Concatenated entities
    are not valid with this command.
    """
    command = f"AADD,{na1},{na2},{na3},{na4},{na5},{na6},{na7},{na8},{na9}"
    return self.run(command, **kwargs)


def aglue(self, na1="", na2="", na3="", na4="", na5="", na6="", na7="",
          na8="", na9="", **kwargs):
    """Generates new areas by "gluing" areas.

    APDL Command: AGLUE

    Parameters
    ----------
    na1, na2, na3, . . . , na9
        Numbers of the areas to be glued.  If NA1 = ALL, all selected areas
        will be glued (NA2 to NA9 will be ignored).  If NA1 = P, graphical
        picking is enabled and all remaining arguments are ignored (valid
        only in the GUI).  A component name may also be substituted for
        NA1.

    Notes
    -----
    Use of the AGLUE command generates new areas by "gluing" input areas.
    The glue operation redefines the input areas so that they share lines
    along their common boundaries.  The new areas encompass the same
    geometry as the original areas.  This operation is only valid if the
    intersection of the input areas are lines along the boundaries of those
    areas.  See the Modeling and Meshing Guide for an illustration.  See
    the BOPTN command for an explanation of the options available to
    Boolean operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be transferred to
    new entities generated.

    The AGLUE command results in the merging of lines and keypoints at the
    common area boundaries. The lines and keypoints of the lower numbered
    area will be kept. This means one must be aware of area numbering when
    multiple AGLUE commands are applied to avoid any "ungluing" of
    geometry.
    """
    command = f"AGLUE,{na1},{na2},{na3},{na4},{na5},{na6},{na7},{na8},{na9}"
    return self.run(command, **kwargs)


def aina(self, na1="", na2="", na3="", na4="", na5="", na6="", na7="",
         na8="", na9="", **kwargs):
    """Finds the intersection of areas.

    APDL Command: AINA

    Parameters
    ----------
    na1, na2, na3, . . . , na9
        Numbers of areas to be intersected.  If NA1 = ALL, NA2 to NA9 are
        ignored and the intersection of all selected areas is found.  If
        NA1 = P, graphical picking is enabled and all remaining arguments
        are ignored (valid only in the GUI).  A component name may also be
        substituted for NA1.

    Notes
    -----
    Finds the common (not pairwise) intersection of areas.  The common
    intersection is defined as the regions shared (in common) by all areas
    listed on this command.  New areas will be generated where the original
    areas intersect.  If the regions of intersection are only lines, new
    lines will be generated instead.  See the Modeling and Meshing Guide
    for an illustration.  See the BOPTN command for the options available
    to Boolean operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be transferred to
    the new entities generated.
    """
    command = f"AINA,{na1},{na2},{na3},{na4},{na5},{na6},{na7},{na8},{na9}"
    return self.run(command, **kwargs)


def ainp(self, na1="", na2="", na3="", na4="", na5="", na6="", na7="",
         na8="", na9="", **kwargs):
    """Finds the pairwise intersection of areas.

    APDL Command: AINP

    Parameters
    ----------
    na1, na2, na3, . . . , na9
        Numbers of areas to be intersected pairwise.  If NA1 = ALL, NA2 to
        NA9 are ignored and the pairwise intersection of all selected areas
        is found.  If NA1 = P, graphical picking is enabled and all
        remaining arguments are ignored (valid only in the GUI).  A
        component name may be substituted for NA1.

    Notes
    -----
    Finds the pairwise intersection of areas.  The pairwise intersection is
    defined as all regions shared by any two or more areas listed on this
    command.  New areas will be generated where the original areas
    intersect pairwise.  If the regions of pairwise intersection are only
    lines, new lines will be generated.  See the Modeling and Meshing Guide
    for an illustration.  See the BOPTN command for the options available
    to Boolean operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be transferred to
    the new entities generated.
    """
    command = f"AINP,{na1},{na2},{na3},{na4},{na5},{na6},{na7},{na8},{na9}"
    return self.run(command, **kwargs)


def ainv(self, na="", nv="", **kwargs):
    """Finds the intersection of an area with a volume.

    APDL Command: AINV

    Parameters
    ----------
    na
        Number of area to be intersected.  If P, graphical picking is
        enabled and all remaining arguments are ignored (valid only in the
        GUI).

    nv
        Number of volume to be intersected.

    Notes
    -----
    New areas will be generated where the areas intersect the volumes.  If
    the regions of intersection are only lines, new lines will be generated
    instead.  See the Modeling and Meshing Guide for an illustration.  See
    the BOPTN command for the options available to Boolean operations.
    Element attributes and solid model boundary conditions assigned to the
    original entities will not be transferred to the new entities
    generated.
    """
    command = f"AINV,{na},{nv}"
    return self.run(command, **kwargs)


def aovlap(self, na1="", na2="", na3="", na4="", na5="", na6="", na7="",
           na8="", na9="", **kwargs):
    """Overlaps areas.

    APDL Command: AOVLAP

    Parameters
    ----------
    na1, na2, na3, . . . , na9
        Numbers of areas to be operated on.  If NA1 = ALL, use all selected
        areas and ignore NA2 to NA9.  If NA1 = P, graphical picking is
        enabled and all remaining arguments are ignored (valid only in the
        GUI).  A component name may also be substituted for NA1.

    Notes
    -----
    Generates new areas which encompass the geometry of all the input
    areas. The new areas are defined by the regions of intersection of the
    input areas, and by the complementary (non-intersecting) regions.  See
    Solid Modeling in the Modeling and Meshing Guide for an illustration.
    This operation is only valid when the region of intersection is an
    area.  See the BOPTN command for an explanation of the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"AOVLAP,{na1},{na2},{na3},{na4},{na5},{na6},{na7},{na8},{na9}"
    return self.run(command, **kwargs)


def aptn(self, na1="", na2="", na3="", na4="", na5="", na6="", na7="",
         na8="", na9="", **kwargs):
    """Partitions areas.

    APDL Command: APTN

    Parameters
    ----------
    na1, na2, na3, . . . , na9
        Numbers of areas to be operated on.  If NA1 = ALL, NA2 to NA9 are
        ignored and all selected areas are used.  If NA1 = P, graphical
        picking is enabled and all remaining arguments are ignored (valid
        only in the GUI).  A component name may be substituted for NA1.

    Notes
    -----
    Partitions areas that intersect.  This command is similar to the
    combined functionality of the ASBA and AOVLAP commands.  If the
    intersection of two or more areas is an area (i.e., planar), new areas
    will be created with boundaries that conform to the area of
    intersection and to the boundaries of the non-intersecting portions of
    the input areas [AOVLAP].  If the intersection is a line (i.e., not
    planar), the areas will be subtracted, or divided, along the line(s) of
    intersection [ASBA].  Both types of intersection can occur during a
    single APTN operation.  Areas that do not intersect will not be
    modified.  See the Modeling and Meshing Guide for an illustration.  See
    the BOPTN command for an explanation of the options available to
    Boolean operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be transferred to
    the new entities generated.
    """
    command = f"APTN,{na1},{na2},{na3},{na4},{na5},{na6},{na7},{na8},{na9}"
    return self.run(command, **kwargs)


def asba(self, na1="", na2="", sepo="", keep1="", keep2="", **kwargs) -> int:
    """Subtracts areas from areas.

    APDL Command: ASBA

    Parameters
    ----------
    na1
        Area (or areas, if picking is used) to be subtracted from.
        If ALL, use all selected areas.  Areas specified in this
        argument are not available for use in the NA2 argument.  A
        component name may also be substituted for NA1.

    na2
        Area (or areas, if picking is used) to subtract.  If ALL,
        use all selected areas (except those included in the NA1
        argument).  A component name may also be substituted for
        NA2.

    sepo
        Behavior if the intersection of the NA1 areas and the NA2 areas is
        a line or lines:

        (blank) - The resulting areas will share line(s) where they touch.

        SEPO - The resulting areas will have separate, but
               coincident line(s) where they touch.

    keep1
        Specifies whether NA1 areas are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NA1 areas after ASBA operation (override
        BOPTN command settings).

        KEEP - Keep NA1 areas after ASBA operation (override BOPTN
        command settings).

    keep2
        Specifies whether NA2 areas are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NA2 areas after ASBA operation (override
        BOPTN command settings).

        KEEP - Keep NA2 areas after ASBA operation (override BOPTN
        command settings).

    Returns
    -------
    int
        Area number of the new area (if applicable)

    Examples
    --------
    Subtract a 0.5 x 0.5 rectangle from a 1 x 1 rectangle.

    >>> anum0 = mapdl.blc4(0, 0, 1, 1)
    >>> anum1 = mapdl.blc4(0.25, 0.25, 0.5, 0.5)
    >>> aout = mapdl.asba(anum0, anum1)
    >>> aout
    3

    Notes
    -----
    Generates new areas by subtracting the regions common to both
    NA1 and NA2 areas (the intersection) from the NA1 areas.  The
    intersection can be an area(s) or line(s).  If the
    intersection is a line and SEPO is blank, the NA1 area is
    divided at the line and the resulting areas will be connected,
    sharing a common line where they touch.  If SEPO is set to
    SEPO, NA1 is divided into two unconnected areas with separate
    lines where they touch.  See Solid Modeling in the Modeling
    and Meshing Guide for an illustration.  See the BOPTN command
    for an explanation of the options available to Boolean
    operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be
    transferred to the new entities generated.  ASBA,ALL,ALL will
    have no effect since all the areas (in NA1) will be
    unavailable as NA2 areas.
    """
    command = f"ASBA,{na1},{na2},{sepo},{keep1},{keep2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def asbl(self, na="", nl="", keepa="", keepl="", **kwargs):
    """Subtracts lines from areas.

    APDL Command: ASBL

    Parameters
    ----------
    na
        Area (or areas, if picking is used) to be subtracted from.  If ALL,
        use all selected areas.  If P, graphical picking is enabled (valid
        only in the GUI) and remaining fields are ignored.  A component
        name may also be substituted for NA.

    nl
        Line (or lines, if picking is used) to subtract.  If ALL, use all
        selected lines.    A component name may also be substituted for NL.

    keepa
        Specifies whether NA areas are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NA areas after ASBL operation (override BOPTN command settings).

        KEEP - Keep NA areas after ASBL operation (override BOPTN command settings).

    keepl
        Specifies whether NL lines are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NL lines after ASBL operation (override BOPTN command settings).

        KEEP - Keep NL lines after ASBL operation (override BOPTN command settings).

    Notes
    -----
    Generates new areas by subtracting the regions common to both the areas
    and lines (the intersection) from the NA areas.  The intersection will
    be a line(s).  See Solid Modeling in the Modeling and Meshing Guide for
    an illustration.  See the BOPTN command for an explanation of the
    options available to Boolean operations.  Element attributes and solid
    model boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"ASBL,{na},{nl},{keepa},{keepl}"
    return self.run(command, **kwargs)


def asbv(self, na="", nv="", sepo="", keepa="", keepv="", **kwargs):
    """Subtracts volumes from areas.

    APDL Command: ASBV

    Parameters
    ----------
    na
        Area (or areas, if picking is used) to be subtracted from.  If ALL,
        use all selected areas.  If P, graphical picking is enabled (valid
        only in the GUI) and remaining fields are ignored.  A component
        name may also be substituted for NA.

    nv
        Volume (or volumes, if picking is used) to subtract.  If ALL, use
        all selected volumes.  A component name may also be substituted for
        NV.

    sepo
        Behavior if the intersection of the areas and the volumes is a line
        or lines:

        (blank) - The resulting areas will share line(s) where they touch.

        SEPO - The resulting areas will have separate, but coincident line(s) where they
               touch.

    keepa
        Specifies whether NA areas are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NA areas after ASBV operation (override BOPTN command settings).

        KEEP - Keep NA areas after ASBV operation (override BOPTN command settings).

    keepv
        Specifies whether NV volumes are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete volumes after ASBV operation (override BOPTN command settings).

        KEEP - Keep volumes after ASBV operation (override BOPTN command settings).

    Notes
    -----
    Generates new areas by subtracting the regions common to both NA areas
    and NV volumes (the intersection) from the NA areas.  The intersection
    can be an area(s) or line(s).  If the intersection is a line and SEPO
    is blank, the NA area is divided at the line and the resulting areas
    will be connected, sharing a common line where they touch.  If SEPO is
    set to SEPO, NA is divided into two unconnected areas with separate
    lines where they touch.  See Solid Modeling in the Modeling and Meshing
    Guide for an illustration.  See the BOPTN command for an explanation of
    the options available to Boolean operations.  Element attributes and
    solid model boundary conditions assigned to the original entities will
    not be transferred to the new entities generated.
    """
    command = f"ASBV,{na},{nv},{sepo},{keepa},{keepv}"
    return self.run(command, **kwargs)


def asbw(self, na="", sepo="", keep="", **kwargs):
    """Subtracts the intersection of the working plane from areas (divides

    APDL Command: ASBW
    areas).

    Parameters
    ----------
    na
        Area (or areas, if picking is used) to be subtracted from.  If NA =
        ALL, use all selected areas.  If NA = P, graphical picking is
        enabled (valid only in the GUI).  A component name may also be
        input for NA.

    sepo
        Behavior of the created boundary.

        (blank) - The resulting areas will share line(s) where they touch.

        SEPO - The resulting areas will have separate, but coincident line(s).

    keep
        Specifies whether NA areas are to be deleted.

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NA areas after ASBW operation (override BOPTN command settings).

        KEEP - Keep NA areas after ASBW operation (override BOPTN command settings).

    Notes
    -----
    Generates new areas by subtracting the intersection of the working
    plane from the NA areas.  The intersection will be a line(s).  The
    working plane must not be in the same plane as the NA areas(s).  If
    SEPO is blank, the NA area is divided at the line and the resulting
    areas will be connected, sharing a common line where they touch.  If
    SEPO is set to SEPO, NA is divided into two unconnected areas with
    separate lines.  The SEPO option may cause unintended consequences if
    any keypoints exist along the cut plane. See Solid Modeling in the
    Modeling and Meshing Guide for an illustration.  See the BOPTN command
    for an explanation of the options available to Boolean operations.
    Element attributes and solid model boundary conditions assigned to the
    original entities will not be transferred to the new entities
    generated.

    Issuing the ASBW command under certain conditions may generate a
    topological degeneracy error. Do not issue the command if:

    A sphere or cylinder has been scaled. (A cylinder must be scaled
    unevenly in the XY plane.)

    A sphere or cylinder has not been scaled but the work plane has been
    rotated.
    """
    command = f"ASBW,{na},{sepo},{keep}"
    return self.run(command, **kwargs)


def boptn(self, lab="", value="", **kwargs):
    """Specifies Boolean operation options.

    APDL Command: BOPTN

    Parameters
    ----------
    lab
        Default/status key:

        DEFA  - Resets settings to default values.

        STAT  - Lists status of present settings.

    value
        Option settings if Lab = KEEP:

        NO  - Delete entities used as input with a Boolean operation (default).  Entities
              will not be deleted if meshed or if attached to a higher
              entity.

        YES  - Keep input solid modeling entities.

    Notes
    -----
    Boolean operations at Revision 5.2 may produce a different number of
    entities than previous revisions of ANSYS.   When running input files
    created at earlier revisions of ANSYS, match the Boolean compatibility
    option (VERSION) to the revision originally used. For instance,  if you
    are running Revision 5.2 and are reading an input file (/INPUT) created
    at Revision 5.1, it is recommended that you set VERSION to RV51 before
    reading the input.

    See the Modeling and Meshing Guide for further details on the functions
    of the RV51 and RV52 labels.

    This command is valid in any processor.
    """
    command = f"BOPTN,{lab},{value}"
    return self.run(command, **kwargs)


def btol(self, ptol="", **kwargs):
    """Specifies the Boolean operation tolerances.

    APDL Command: BTOL

    Parameters
    ----------
    ptol
        Point coincidence tolerance.  Points within this distance to each
        other will be assumed to be coincident during Boolean operations.
        Loosening the tolerance will increase the run time and storage
        requirements, but will allow more Boolean intersections to succeed.
        Defaults to 0.10E-4.

    Notes
    -----
    Use BTOL,DEFA to reset the setting to its default value.  Use BTOL,STAT
    to list the status of the present setting.
    """
    command = f"BTOL,{ptol}"
    return self.run(command, **kwargs)


def lcsl(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", nl7="",
         nl8="", nl9="", **kwargs):
    """Divides intersecting lines at their point(s) of intersection.

    APDL Command: LCSL

    Parameters
    ----------
    nl1, nl2, nl3, . . . , nl9
        Numbers of lines to be intersected.  If NL1 = ALL, NL2 to NL9 are
        ignored and the intersection of all selected lines is found.  If
        NL1 = P, use graphical picking to specify lines (NL2 to NL9 are
        ignored).

    Notes
    -----
    Divides intersecting (classifies) lines at their point(s) of
    intersection.  The original lines (and their corresponding keypoint(s))
    will be deleted by default.  See the BOPTN command for the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"LCSL,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{nl7},{nl8},{nl9}"
    return self.run(command, **kwargs)


def lglue(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", nl7="",
          nl8="", nl9="", **kwargs):
    """Generates new lines by "gluing" lines.

    APDL Command: LGLUE

    Parameters
    ----------
    nl1, nl2, nl3, . . . , nl9
        Numbers of the lines to be glued.  If NL1 = ALL, all selected lines
        will be glued (NL2 to NL9 will be ignored).  If NL1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for NL1.

    Notes
    -----
    Use of the LGLUE command generates new lines by "gluing" input lines.
    The glue operation redefines the input lines so that they share
    keypoints at their common ends.  The new lines encompass the same
    geometry as the original lines.  This operation is only valid if the
    intersections of the input lines are keypoints at the ends of those
    lines.  See the Modeling and Meshing Guide for an illustration.  See
    the BOPTN command for an explanation of the options available to
    Boolean operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be transferred to
    the new entities generated.

    The LGLUE command results in the merging of keypoints at the common end
    of the lines. The keypoints of the lower numbered line will be kept.
    This means one must be aware of line numbering when multiple LGLUE
    commands are applied to avoid any "ungluing" of geometry.
    """
    command = f"LGLUE,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{nl7},{nl8},{nl9}"
    return self.run(command, **kwargs)


def lina(self, nl="", na="", **kwargs):
    """Finds the intersection of a line with an area.

    APDL Command: LINA

    Parameters
    ----------
    nl
        Number of line to be intersected.  If NL = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).

    na
        Number of area to be intersected.

    Notes
    -----
    Finds the intersection of a line with an area.  New lines will be
    generated where the lines intersect the areas.  If the regions of
    intersection are only points, new keypoints will be generated instead.
    See the Modeling and Meshing Guide for an illustration.  See the BOPTN
    command for the options available to Boolean operations.  Element
    attributes and solid model boundary conditions assigned to the original
    entities will not be transferred to the new entities generated.
    """
    command = f"LINA,{nl},{na}"
    return self.run(command, **kwargs)


def linl(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", nl7="",
         nl8="", nl9="", **kwargs):
    """Finds the common intersection of lines.

    APDL Command: LINL

    Parameters
    ----------
    nl1, nl2, nl3, . . . , nl9
        Numbers of lines to be intersected.  If NL1 = ALL, find the
        intersection of all selected lines and NL2 to NL9 are ignored.  If
        NL1 = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NL1.

    Notes
    -----
    Finds the common (not pairwise) intersection of lines.  The common
    intersection is defined as the regions shared (in common) by all lines
    listed on this command.  New lines will be generated where the original
    lines intersect.  If the regions of intersection are only points, new
    keypoints will be generated instead.  See the Modeling and Meshing
    Guide for an illustration.  See the BOPTN command for the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"LINL,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{nl7},{nl8},{nl9}"
    return self.run(command, **kwargs)


def linp(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", nl7="",
         nl8="", nl9="", **kwargs):
    """Finds the pairwise intersection of lines.

    APDL Command: LINP

    Parameters
    ----------
    nl1, nl2, nl3, . . . , nl9
        Numbers of lines to be intersected pairwise.  If NL1 =  ALL, find
        the pairwise intersection of all selected lines and NL2 to NL9 are
        ignored.  If NL1 = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).  A
        component name may be substituted for NL1.

    Notes
    -----
    Finds the pairwise intersection of lines.  The pairwise intersection is
    defined as any and all regions shared by at least two lines listed on
    this command.  New lines will be generated where the original lines
    intersect pairwise.  If the regions of pairwise intersection are only
    points, new keypoints will be generated.  See the Modeling and Meshing
    Guide for an illustration.  See the BOPTN command for the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"LINP,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{nl7},{nl8},{nl9}"
    return self.run(command, **kwargs)


def linv(self, nl="", nv="", **kwargs):
    """Finds the intersection of a line with a volume.

    APDL Command: LINV

    Parameters
    ----------
    nl
        Number of line to be intersected.  If NL = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).

    nv
        Number of volume to be intersected.

    Notes
    -----
    Finds the intersection of a line with a volume.  New lines will be
    generated where the lines intersect the volumes.  If the regions of
    intersection are only points, new keypoints will be generated instead.
    See the Modeling and Meshing Guide for an illustration.  See the BOPTN
    command for the options available to Boolean operations.  Element
    attributes and solid model boundary conditions assigned to the original
    entities will not be transferred to the new entities generated.
    """
    command = f"LINV,{nl},{nv}"
    return self.run(command, **kwargs)


def lovlap(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", nl7="",
           nl8="", nl9="", **kwargs):
    """Overlaps lines.

    APDL Command: LOVLAP

    Parameters
    ----------
    nl1, nl2, nl3, . . . , nl9
        Numbers of lines to be overlapped.  If NL1 = ALL, NL2 to NL9 are
        ignored and all selected lines are overlapped.  If NL1 = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NL1.

    Notes
    -----
    Overlaps lines. Generates new lines which encompass the geometry of all
    the input lines.  The new lines are defined by the regions of
    intersection of the input lines, and by the complementary (non-
    intersecting) regions.  See the Modeling and Meshing Guide for an
    illustration.  This operation is only valid when the region of
    intersection is a line.  See the BOPTN command for an explanation of
    the options available to Boolean operations.  Element attributes and
    solid  model boundary conditions assigned to the original entities will
    not be transferred to the new entities generated.
    """
    command = f"LOVLAP,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{nl7},{nl8},{nl9}"
    return self.run(command, **kwargs)


def lptn(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", nl7="",
         nl8="", nl9="", **kwargs):
    """Partitions lines.

    APDL Command: LPTN

    Parameters
    ----------
    nl1, nl2, nl3, . . . , nl9
        Numbers of lines to be operated on.  If NL1 = ALL, NL2 to NL9 are
        ignored all selected lines are used.  If NL1 = P, graphical picking
        is enabled and all remaining command fields are ignored (valid only
        in the GUI).  A component name may be substituted for NL1.

    Notes
    -----
    Partitions lines.  Generates new lines which encompass the geometry of
    all the input lines.  The new lines are defined by both the regions of
    intersection of the input lines and the complementary (non-
    intersecting) regions.  See the Modeling and Meshing Guide for an
    illustration. See the BOPTN command for an explanation of the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"LPTN,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{nl7},{nl8},{nl9}"
    return self.run(command, **kwargs)


def lsba(self, nl="", na="", sepo="", keepl="", keepa="", **kwargs):
    """Subtracts areas from lines.

    APDL Command: LSBA

    Parameters
    ----------
    nl
        Line (or lines, if picking is used) to be subtracted from.  If ALL,
        use all selected lines.  If NL = P, graphical picking is enabled
        and all remaining command fields are ignored (valid only in the
        GUI).  A component name may also be substituted for NL.

    na
        Area (or areas, if picking is used) to be subtracted.  If ALL, use
        all selected areas.  A component name may also be substituted for
        NA.

    sepo
        Behavior if the intersection of the lines and the areas is a
        keypoint or keypoints:

        (blank) - The resulting lines will share keypoint(s) where they touch.

        SEPO - The resulting lines will have separate, but coincident keypoint(s) where they
               touch.

    keepl
        Specifies whether NL lines are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NL lines after LSBA operation (override BOPTN command settings).

        KEEP - Keep NL lines after LSBA operation (override BOPTN command settings).

    keepa
        Specifies whether NA areas are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete areas after LSBA operation (override BOPTN command settings).

        KEEP - Keep areas after LSBA operation (override BOPTN command settings).

    Notes
    -----
    Generates new lines by subtracting the regions common to both NL lines
    and NA areas (the intersection) from the NL lines.  The intersection
    can be a line(s) or keypoint(s).  If the intersection is a keypoint and
    SEPO is blank, the NL line is divided at the keypoint and the resulting
    lines will be connected, sharing a common keypoint where they touch.
    If SEPO is set to SEPO, NL is divided into two unconnected lines with
    separate keypoints where they touch.  See the Modeling and Meshing
    Guide for an illustration.  See the BOPTN command for an explanation of
    the options available to Boolean operations.  Element attributes and
    solid model boundary conditions assigned to the original entities will
    not be transferred to the new entities generated.
    """
    command = f"LSBA,{nl},{na},{sepo},{keepl},{keepa}"
    return self.run(command, **kwargs)


def lsbl(self, nl1="", nl2="", sepo="", keep1="", keep2="", **kwargs):
    """Subtracts lines from lines.

    APDL Command: LSBL

    Parameters
    ----------
    nl1
        Line (or lines, if picking is used) to be subtracted from.  If ALL,
        use all selected lines.  Lines specified in this argument are not
        available for use in the NL2 argument.  If P, graphical picking is
        enabled (valid only in the GUI) and all remaining fields are
        ignored.  A component name may also be substituted for NL1.

    nl2
        Line (or lines, if picking is used) to subtract.  If ALL, use all
        selected lines (except those included in the NL1 argument).  A
        component name may also be substituted for NL2.

    sepo
        Behavior if the intersection of the NL1 lines and the NL2 lines is
        a keypoint or keypoints:

        (blank) - The resulting lines will share keypoint(s) where they touch.

        SEPO - The resulting lines will have separate, but coincident keypoint(s) where they
               touch.

    keep1
        Specifies whether NL1 lines are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NL1 lines after LSBL operation (override BOPTN command settings).

        KEEP - Keep NL1 lines after LSBL operation (override BOPTN command settings).

    keep2
        Specifies whether NL2 lines are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NL2 lines after LSBL operation (override BOPTN command settings).

        KEEP - Keep NL2 lines after LSBL operation (override BOPTN command settings).

    Notes
    -----
    Generates new lines by subtracting the regions common to both NL1 and
    NL2 lines (the intersection) from the NL1 lines.  The intersection can
    be a line(s) or point(s).  If the intersection is a point and SEPO is
    blank, the NL1 line is divided at the point and the resulting lines
    will be connected, sharing a common keypoint where they touch.  If SEPO
    is set to SEPO, NL1 is divided into two unconnected lines with separate
    keypoints where they touch.  See the Modeling and Meshing Guide for an
    illustration.  See the BOPTN command for an explanation of the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.  LSBL,ALL,ALL will have no
    effect since all the lines (in NL1) will be unavailable as NL2 lines.
    """
    command = f"LSBL,{nl1},{nl2},{sepo},{keep1},{keep2}"
    return self.run(command, **kwargs)


def lsbv(self, nl="", nv="", sepo="", keepl="", keepv="", **kwargs):
    """Subtracts volumes from lines.

    APDL Command: LSBV

    Parameters
    ----------
    nl
        Line (or lines, if picking is used) to be subtracted from.  If ALL,
        use all selected lines.  If NL = P, graphical picking is enabled
        and all remaining command fields are ignored (valid only in the
        GUI).  A component name may also be substituted for NL.

    nv
        Volume (or volumes, if picking is used) to be subtracted.  If ALL,
        use all selected volumes.  A component name may also be substituted
        for NV.

    sepo
        Behavior if the intersection of the NL lines and the NV volumes is
        a keypoint or keypoints:

        (blank) - The resulting lines will share keypoint(s) where they touch.

        SEPO - The resulting lines will have separate, but coincident keypoint(s) where they
               touch.

    keepl
        Specifies whether NL lines are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NL lines after LSBV operation (override BOPTN command settings).

        KEEP - Keep NL lines after LSBV operation (override BOPTN command settings).

    keepv
        Specifies whether NV volumes are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NV volumes after LSBV operation (override BOPTN command settings).

        KEEP - Keep NV volumes after LSBV operation (override BOPTN command settings).

    Notes
    -----
    Generates new lines by subtracting the regions common to both NL lines
    and NV volumes (the intersection) from the NL lines.  The intersection
    can be a line(s) or point(s).  If the intersection is a point and SEPO
    is blank, the NL1 line is divided at the point and the resulting lines
    will be connected, sharing a common keypoint where they touch.  If SEPO
    is set to SEPO, NL1 is divided into two unconnected lines with separate
    keypoints where they touch.  See the Modeling and Meshing Guide for an
    illustration.  See the BOPTN command for an explanation of the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.  LSBL,ALL,ALL will have no
    effect since all the lines (in NL1) will be unavailable as NL2 lines.
    """
    command = f"LSBV,{nl},{nv},{sepo},{keepl},{keepv}"
    return self.run(command, **kwargs)


def lsbw(self, nl="", sepo="", keep="", **kwargs):
    """Subtracts the intersection of the working plane from lines (divides

    APDL Command: LSBW
    lines).

    Parameters
    ----------
    nl
        Line (or lines, if picking is used) to be subtracted from.  If NL =
        ALL, use all selected lines.  If NL = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be input for NL.

    sepo
        Behavior of the created boundary.

        (blank) - The resulting lines will share keypoint(s) where they touch.

        SEPO - The resulting lines will have separate, but coincident keypoint(s).

    keep
        Specifies whether NL lines are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NL lines after LSBW operation (override BOPTN command settings).

        KEEP - Keep NL lines after LSBW operation (override BOPTN command settings).

    Notes
    -----
    Generates new lines by subtracting the intersection of the working
    plane from the NL lines.  The intersection will be a keypoint(s).  The
    working plane must not be in the same plane as the NL line(s).  If SEPO
    is blank, the NL line is divided and the resulting lines will be
    connected, sharing a common keypoint where they touch.  If SEPO is set
    to SEPO, NL is divided into two unconnected lines with separate
    keypoints.  See the Modeling and Meshing Guide for an illustration.
    See the BOPTN command for an explanation of the options available to
    Boolean operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be transferred to
    the new entities generated.  Areas that completely contain the input
    lines will be updated if the lines are divided by this operation.
    """
    command = f"LSBW,{nl},{sepo},{keep}"
    return self.run(command, **kwargs)


def vadd(self, nv1="", nv2="", nv3="", nv4="", nv5="", nv6="", nv7="",
         nv8="", nv9="", **kwargs):
    """Adds separate volumes to create a single volume.

    APDL Command: VADD

    Parameters
    ----------
    nv1, nv2, nv3, . . . , nv9
        Numbers of volumes to be added.  If NV1 = ALL, add all selected
        volumes and ignore NV2 to NV9.  If NV1 = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for NV1.

    Notes
    -----
    Adds separate volumes to create a single volume.  The original volumes
    (and their corresponding areas, lines and keypoints) will be deleted by
    default [BOPTN].  See the BOPTN command for the options available to
    Boolean operations.  Element attributes and solid model boundary
    conditions assigned to the original entities will not be transferred to
    the new entities generated.  Concatenated entities are not valid with
    this command.
    """
    command = f"VADD,{nv1},{nv2},{nv3},{nv4},{nv5},{nv6},{nv7},{nv8},{nv9}"
    return self.run(command, **kwargs)


def vglue(self, nv1="", nv2="", nv3="", nv4="", nv5="", nv6="", nv7="",
          nv8="", nv9="", **kwargs):
    """Generates new volumes by "gluing" volumes.

    APDL Command: VGLUE

    Parameters
    ----------
    nv1, nv2, nv3, . . . , nv9
        Numbers of the volumes to be glued.  If NV1 = ALL, all selected
        volumes will be glued (NV2 to NV9 will be ignored).  If NV1 = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NV1.

    Notes
    -----
    Use of the VGLUE command generates new volumes by "gluing" input
    volumes.  The glue operation redefines the input volumes so that they
    share areas along their common boundaries.  The new volumes encompass
    the same geometry as the original volumes.  This operation is only
    valid if the intersections of the input volumes are areas along the
    boundaries of those volumes.  See the Modeling and Meshing Guide for an
    illustration.  See the BOPTN command for an explanation of the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.

    The VGLUE command results in the merging of areas, lines, and keypoints
    at the common volume boundaries. The areas, lines, and keypoints of the
    lower numbered volume will be kept. This means one must be aware of
    volume numbering when multiple VGLUE commands are applied to avoid any
    "ungluing" of geometry.
    """
    command = f"VGLUE,{nv1},{nv2},{nv3},{nv4},{nv5},{nv6},{nv7},{nv8},{nv9}"
    return self.run(command, **kwargs)


def vinp(self, nv1="", nv2="", nv3="", nv4="", nv5="", nv6="", nv7="",
         nv8="", nv9="", **kwargs):
    """Finds the pairwise intersection of volumes.

    APDL Command: VINP

    Parameters
    ----------
    nv1, nv2, nv3, . . . , nv9
        Numbers of volumes to be intersected pairwise.  If NV1 = ALL, NV2
        to NV9 are ignored and the pairwise intersection of all selected
        volumes is found.  If NV1 = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).  A
        component name may also be substituted for NV1.

    Notes
    -----
    Finds the pairwise intersection of volumes.  The pairwise intersection
    is defined as all regions shared by any two or more volumes listed on
    this command.  New volumes will be generated where the original volumes
    intersect pairwise.  If the regions of pairwise intersection are only
    areas, new areas will be generated.  See the Modeling and Meshing Guide
    for an illustration.  See the BOPTN command for an explanation of the
    options available to Boolean operations.  Element attributes and solid
    model boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"VINP,{nv1},{nv2},{nv3},{nv4},{nv5},{nv6},{nv7},{nv8},{nv9}"
    return self.run(command, **kwargs)


def vinv(self, nv1="", nv2="", nv3="", nv4="", nv5="", nv6="", nv7="",
         nv8="", nv9="", **kwargs):
    """Finds the intersection of volumes.

    APDL Command: VINV

    Parameters
    ----------
    nv1, nv2, nv3, . . . , nv9
        Numbers of volumes to be intersected.  If NV1 = ALL, NV2 to NV9 are
        ignored, and the intersection of all selected volumes is found.  If
        NV1 = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NV1.

    Notes
    -----
    Finds the common (not pairwise) intersection of volumes.  The common
    intersection is defined as the regions shared (in common) by all
    volumes listed on this command.  New volumes will be generated where
    the original volumes intersect.  If the regions of intersection are
    only areas, new areas will be generated instead.  See the Modeling and
    Meshing Guide for an illustration.  See the BOPTN command for an
    explanation of the options available to Boolean operations.  Element
    attributes and solid model boundary conditions assigned to the original
    entities will not be transferred to the new entities generated.
    """
    command = f"VINV,{nv1},{nv2},{nv3},{nv4},{nv5},{nv6},{nv7},{nv8},{nv9}"
    return self.run(command, **kwargs)


def vovlap(self, nv1="", nv2="", nv3="", nv4="", nv5="", nv6="", nv7="",
           nv8="", nv9="", **kwargs):
    """Overlaps volumes.

    APDL Command: VOVLAP

    Parameters
    ----------
    nv1, nv2, nv3, . . . , nv9
        Numbers of volumes to be operated on.  If NV1 = ALL, NV2 to NV9 are
        ignored and all selected volumes are used.  If NV1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for NV1.

    Notes
    -----
    Overlaps volumes. Generates new volumes which encompass the geometry of
    all  the input volumes.  The new volumes are defined by the regions of
    intersection  of the input volumes, and by the complementary (non-
    intersecting) regions.   See the Modeling and Meshing Guide for an
    illustration.  This operation is only valid  when the region of
    intersection is a volume.  See the BOPTN command for an explanation of
    the options available to Boolean operations.  Element attributes and
    solid model boundary conditions assigned to the original entities will
    not be transferred to the new entities generated.
    """
    command = f"VOVLAP,{nv1},{nv2},{nv3},{nv4},{nv5},{nv6},{nv7},{nv8},{nv9}"
    return self.run(command, **kwargs)


def vptn(self, nv1="", nv2="", nv3="", nv4="", nv5="", nv6="", nv7="",
         nv8="", nv9="", **kwargs):
    """Partitions volumes.

    APDL Command: VPTN

    Parameters
    ----------
    nv1, nv2, nv3, . . . , nv9
        Numbers of volumes to be operated on.  If NV1 = ALL, NV2 to NV9 are
        ignored and all selected volumes are used.  If NV1  = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for NV1.

    Notes
    -----
    Partitions volumes.  Generates new volumes which encompass the geometry
    of all the input volumes.  The new volumes are defined by the regions
    of intersection of the input volumes, and by the complementary (non-
    intersecting) regions.  See the Modeling and Meshing Guide for an
    illustration. See the BOPTN command for an explanation of the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"VPTN,{nv1},{nv2},{nv3},{nv4},{nv5},{nv6},{nv7},{nv8},{nv9}"
    return self.run(command, **kwargs)


def vsba(self, nv="", na="", sepo="", keepv="", keepa="", **kwargs):
    """Subtracts areas from volumes.

    APDL Command: VSBA

    Parameters
    ----------
    nv
        Volume (or volumes, if picking is used) to be subtracted from.  If
        ALL, use all selected volumes.  If P, graphical picking is enabled
        (valid only in the GUI) and remaining fields are ignored.  A
        component name may also be substituted for NV.

    na
        Area (or areas, if picking is used) to subtract.  If ALL, use all
        selected areas.  A component name may also be substituted for NA.

    sepo
        Behavior of the touching boundary:

        (blank) - The resulting volumes will share area(s) where they touch.

        SEPO - The resulting volumes will have separate, but coincident area(s) where they
               touch.

    keepv
        Specifies whether NV volumes are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NV volumes after VSBA operation (override BOPTN command settings).

        KEEP - Keep NV volumes after VSBA operation (override BOPTN command settings).

    keepa
        Specifies whether NA areas are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NA areas after VSBA operation (override BOPTN command settings).

        KEEP - Keep NA areas after VSBA operation (override BOPTN command settings).

    Notes
    -----
    Generates new volumes by subtracting the regions common to both the
    volumes and areas (the intersection) from the NV volumes.  The
    intersection will be an area(s).  If SEPO is blank, the volume is
    divided at the area and the resulting volumes will be connected,
    sharing a common area where they touch.  If SEPO is set to SEPO, the
    volume is divided into two unconnected volumes with separate areas
    where they touch.  See the Modeling and Meshing Guide for an
    illustration.  See the BOPTN command for an explanation of the options
    available to Boolean operations.  Element attributes and solid model
    boundary conditions assigned to the original entities will not be
    transferred to the new entities generated.
    """
    command = f"VSBA,{nv},{na},{sepo},{keepv},{keepa}"
    return self.run(command, **kwargs)


def vsbv(self, nv1="", nv2="", sepo="", keep1="", keep2="", **kwargs):
    """Subtracts volumes from volumes.

    APDL Command: VSBV

    Parameters
    ----------
    nv1
        Volume (or volumes, if picking is used) to be subtracted
        from.  If ALL, use all selected volumes.  Volumes
        specified in set NV2 are removed from set NV1.  If P,
        graphical picking is enabled (valid only in the GUI) and
        remaining fields are ignored.  A component name may also
        be substituted for NV1.

    nv2
        Volume (or volumes, if picking is used) to subtract.  If
        ALL, use all selected volumes (except those included in
        the NV1 argument).  A component name may also be
        substituted for NV2.

    sepo
        Behavior if the intersection of the NV1 volumes and the
        NV2 volumes is an area or areas:

        (blank) - The resulting volumes will share area(s) where they touch.

        SEPO - The resulting volumes will have separate, but
               coincident area(s) where they touch.

    keep1
        Specifies whether NV1 volumes are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NV1 volumes after VSBV operation (override
                 BOPTN command settings).

        KEEP - Keep NV1 volumes after VSBV operation (override
               BOPTN command settings).

    keep2
        Specifies whether NV2 volumes are to be deleted:

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NV2 volumes after VSBV operation (override
                 BOPTN command settings).

        KEEP - Keep NV2 volumes after VSBV operation (override
               BOPTN command settings).

    Notes
    -----
    Generates new volumes by subtracting the regions common to
    both NV1 and NV2 volumes (the intersection) from the NV1
    volumes.  The intersection can be a volume(s) or area(s).  If
    the intersection is an area and SEPO is blank, the NV1 volume
    is divided at the area and the resulting volumes will be
    connected, sharing a common area where they touch.  If SEPO is
    set to SEPO, NV1 is divided into two unconnected volumes with
    separate areas where they touch.  See the Modeling and Meshing
    Guide for an illustration.  See the BOPTN command for an
    explanation of the options available to Boolean operations.
    Element attributes and solid model boundary conditions
    assigned to the original entities will not be transferred to
    the new entities generated.  VSBV,ALL,ALL will have no effect
    because all the volumes in set NV1will have been moved to set
    NV2.
    """
    command = f"VSBV,{nv1},{nv2},{sepo},{keep1},{keep2}"
    return self.run(command, **kwargs)


def vsbw(self, nv="", sepo="", keep="", **kwargs):
    """Subtracts intersection of the working plane from volumes (divides

    APDL Command: VSBW
    volumes).

    Parameters
    ----------
    nv
        Volume (or volumes, if picking is used) to be subtracted from.  If
        NV = ALL, use all selected volumes.  If NV = P, graphical picking
        is enabled (valid only in the GUI).  A component name may also be
        input for NV.

    sepo
        Behavior of the created boundary.

        (blank) - The resulting volumes will share area(s) where they touch.

        SEPO - The resulting volumes will have separate, but coincident area(s).

    keep
        Specifies whether NV volumes are to be deleted.

        (blank) - Use the setting of KEEP on the BOPTN command.

        DELETE - Delete NV volumes after VSBW operation (override BOPTN command settings).

        KEEP - Keep NV volumes after VSBW operation (override BOPTN command settings).

    Notes
    -----
    Generates new volumes by subtracting the intersection of the working
    plane from the NV volumes.  The intersection will be an area(s).  If
    SEPO is blank, the volume is divided at the area and the resulting
    volumes will be connected, sharing a common area where they touch.  If
    SEPO is set to SEPO, the volume is divided into two unconnected volumes
    with separate areas.  The SEPO option may cause unintended consequences
    if any keypoints exist along the cut plane. See the Modeling and
    Meshing Guide for an illustration.  See the BOPTN command for an
    explanation of the options available to Boolean operations.  Element
    attributes and solid model boundary conditions assigned to the original
    entities will not be transferred to the new entities generated.

    Issuing the VSBW command under certain conditions may generate a
    topological degeneracy error. Do not issue the command if:

    A sphere or cylinder has been scaled. (A cylinder must be scaled
    unevenly in the XY plane.)

    A sphere or cylinder has not been scaled but the work plane has been
    rotated.
    """
    command = f"VSBW,{nv},{sepo},{keep}"
    return self.run(command, **kwargs)



def f(self, node="", lab="", value="", value2="", nend="", ninc="",
      **kwargs):
    """Specifies force loads at nodes.

    APDL Command: F

    Parameters
    ----------
    node
        Node at which force is to be specified.  If ALL, NEND and NINC are
        ignored and forces are applied to all selected nodes [NSEL]. If
        NODE = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NODE.

    lab
        Valid force label.  Structural labels:  FX, FY, or FZ (forces); MX,
        MY, or MZ (moments).  Thermal labels:  HEAT, HBOT, HE2, HE3, . . .,
        HTOP (heat flow).  Fluid labels:  FLOW (fluid flow).   Electric
        labels:  AMPS (current flow), CHRG (electric charge).  Magnetic
        labels:  FLUX (magnetic flux);  CSGX, CSGY, or CSGZ (magnetic
        current segments).  Diffusion labels: RATE (diffusion flow rate).

    value
        Force value or table name reference for specifying tabular boundary
        conditions.  To specify a table, enclose the table name in percent
        signs (%), e.g., F, NODE,HEAT,%tabname%).  Use the *DIM command to
        define a table.

    value2
        Second force value (if any).  If the analysis type and the force
        allow a complex input, VALUE (above) is the real component and
        VALUE2 is the imaginary component.

    nend, ninc
        Specifies the same values of force at the nodes ranging from NODE
        to NEND (defaults to NODE), in steps of NINC (defaults to 1).

    Notes
    -----
    The available force loads per node correspond to the degrees of freedom
    listed under "Degrees of Freedom" in the input table for each element
    type in the Element Reference.  If both a force and a constrained
    degree of freedom [D] are specified at the same node, the constraint
    takes precedence.  Forces are defined in the nodal coordinate system.
    The positive directions of structural forces and moments are along and
    about the positive nodal axis directions.  The node and the degree of
    freedom label corresponding to the force must be selected [NSEL,
    DOFSEL].

    For hydrostatic fluid elements (HSFLD241 and HSFLD242), DVOL is used to
    specify fluid mass flow rate (with units of mass/time) at the pressure
    node. This allows fluid to be added or taken out of the fluid elements
    sharing the pressure node. A fluid density must also be specified (via
    the MP command or TB command) to apply a volume change corresponding to
    the prescribed fluid mass flow rate.

    Tabular boundary conditions (VALUE = %tabname%) are available only for
    the following labels: Fluid (FLOW), Electric (AMPS), Structural force
    (FX, FY, FZ, MX, MY, MZ), and Thermal (HEAT, HBOT, HE2, HE3, . . .,
    HTOP). Tabular boundary conditions are valid only in static
    (ANTYPE,STATIC), full transient (ANTYPE,TRANS), full harmonic (ANTYPE,
    HARMIC), modal superposition harmonic and modal superposition transient
    analyses.

    This command is also valid in PREP7.
    """
    command = f"F,{node},{lab},{value},{value2},{nend},{ninc}"
    return self.run(command, **kwargs)


def fcum(self, oper="", rfact="", ifact="", **kwargs):
    """Specifies that force loads are to be accumulated.

    APDL Command: FCUM

    Parameters
    ----------
    oper
        Accumulation key:

        REPL - Subsequent values replace the previous values (default).

        ADD - Subsequent values are added to the previous values.

        IGNO - Subsequent values are ignored.

    rfact
        Scale factor for the real component.  Zero (or blank) defaults to
        1.0.  Use a small number for a zero scale factor.

    ifact
        Scale factor for the imaginary component.  Zero (or blank) defaults
        to 1.0.  Use a small number for a zero scale factor.

    Notes
    -----
    Allows repeated force load (force, heat flow, etc.) values to be
    replaced, added, or ignored.  Operations apply to the selected nodes
    [NSEL]. and the force labels corresponding to the selected force labels
    [DOFSEL].  The operations occur when the next force specifications are
    defined.  For example, issuing the command F,1,FX,250 after a previous
    F,1,FX,200 causes the current value of the force on node 1 in the
    x-direction to be 450 with the add operation, 250 with the replace
    operation, or 200 with the ignore operation.  Scale factors are also
    available to multiply the next value before the add or replace
    operation.  A scale factor of 2.0 with the previous "add" example
    results in a force of 700.  Scale factors are applied even if no
    previous values exist.  Issue FCUM,STAT to show the current label,
    operation, and scale factors.  Solid model boundary conditions are not
    affected by this command, but boundary conditions on the FE model are
    affected.

    Note:: : FE boundary conditions may still be overwritten by existing
    solid model boundary conditions if a subsequent boundary condition
    transfer occurs.

    FCUM does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"FCUM,{oper},{rfact},{ifact}"
    return self.run(command, **kwargs)


def fdele(self, node="", lab="", nend="", ninc="", lkey="", **kwargs):
    """Deletes force loads on nodes.

    APDL Command: FDELE

    Parameters
    ----------
    node
        Node for which force is to be deleted.  If ALL, NEND and NINC are
        ignored and forces are deleted on all selected nodes [NSEL].  If
        NODE = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NODE.

    lab
        Valid force label.  If ALL, use all appropriate labels.  Structural
        labels:  FX, FY, or FZ (forces); MX, MY, or MZ (moments).  Thermal
        labels:  HEAT, HBOT, HE2, HE3, . . ., HTOP (heat flow).  Fluid
        labels:  FLOW (fluid flow).  Electric labels:  AMPS (current flow),
        CHRG (electric charge).  Magnetic labels:  FLUX (magnetic flux);
        CSGX, CSGY, or CSGZ (magnetic current segments).  Diffusion labels:
        RATE (diffusion flow rate).

    nend, ninc
        Delete forces from NODE to NEND (defaults to NODE) in steps of NINC
        (defaults to 1).

    lkey
        Lock key:

        (blank) - The DOF is not locked (default).

        FIXED - Displacement on the specified degrees of freedom (Lab) is locked. The program
                prescribes the degree of freedom to the "current" relative
                displacement value in addition to deleting the force. If a
                displacement constraint (for example, D command) is applied
                in conjunction with this option, the actual applied
                displacement will be ramped during the next load step. The
                displacement is ramped from the current value to the newly
                defined value. This option is only valid for the following
                labels: FX, FY, FZ, MX, MY, MZ. This option is intended
                primarily for use in the ANSYS Workbench interface to apply
                an increment length adjustment (bolt pretension loading).

    Notes
    -----
    The node and the degree of freedom label corresponding to the force
    must be selected [NSEL, DOFSEL].

    This command is also valid in PREP7.
    """
    command = f"FDELE,{node},{lab},{nend},{ninc},{lkey}"
    return self.run(command, **kwargs)


def fj(self, elem="", label="", value="", **kwargs):
    """Specify forces or moments on the components of the relative motion of a

    APDL Command: FJ
    joint element.

    Parameters
    ----------
    elem
        Element number or ALL to specify all joint elements.

    label
        Valid labels:

        FX - Force in local x direction.

        FY - Force in local y direction.

        FZ - Force in local z direction.

        MX - Moment about local x axis.

        MY - Moment about local y axis.

        MZ - Moment about local z axis.

    value
        Value of the label.

    Notes
    -----
    Valid for MPC184 (joint options in KEYOPT(1)).

    See FJDELE for information on deleting forces and moments.
    """
    command = f"FJ,{elem},{label},{value}"
    return self.run(command, **kwargs)


def fjdele(self, elem="", lab="", **kwargs):
    """Deletes forces (or moments) on the components of the relative motion of

    APDL Command: FJDELE
    a joint element.

    Parameters
    ----------
    elem
        Element number, or ALL. (leaving this blank defaults to ALL)

    lab
        Valid labels are:

        FX - Force in local x direction.

        FY - Force in local y direction.

        FZ - Force in local z direction.

        MX - Moment about local x axis.

        MY - Moment about local y axis.

        MZ - Moment about local z axis.

        ALL, or (blank) - Delete all valid forces or moments.

    Notes
    -----
    Valid for MPC184 (joint options in KEYOPT(1)).

    See FJ for information on specifying forces (or moments).
    """
    command = f"FJDELE,{elem},{lab}"
    return self.run(command, **kwargs)


def fjlist(self, elem="", **kwargs):
    """Lists forces and moments applied on joint elements.

    APDL Command: FJLIST

    Parameters
    ----------
    elem
        Element number or ALL (or blank). Lists joint element forces and
        moments on the specified element(s).

    Notes
    -----
    Valid for MPC184 joint elements. See FJ for information on specifying
    forces and moments.
    """
    command = f"FJLIST,{elem}"
    return self.run(command, **kwargs)


def flist(self, node1="", node2="", ninc="", **kwargs):
    """Lists force loads on the nodes.

    APDL Command: FLIST

    Parameters
    ----------
    node1, node2, ninc
        List forces for nodes NODE1 to NODE2 (defaults to NODE1) in steps
        of NINC (defaults to 1).  If ALL, list for all selected nodes
        [NSEL] and NODE2 and NINC  are ignored (default).  If NODE1 = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NODE1.

    Notes
    -----
    Listing applies to the selected nodes [NSEL] and the selected force
    labels [DOFSEL].

    Caution:: : A list containing a node number that is larger than the
    maximum defined node (NODE2), could deplete the system memory and
    produce unpredictable results.

    This command is valid in any processor.
    """
    command = f"FLIST,{node1},{node2},{ninc}"
    return self.run(command, **kwargs)


def fssect(self, rho="", nev="", nlod="", kbr="", **kwargs):
    """Calculates and stores total linearized stress components.

    APDL Command: FSSECT

    Parameters
    ----------
    rho
        In-plane (X-Y) average radius of curvature of the inside and
        outside surfaces of an axisymmetric section.  If zero (or blank), a
        plane or 3-D structure is assumed.  If nonzero, an axisymmetric
        structure is assumed.  Use a suitably large number (see the
        Mechanical APDL Theory Reference) or use -1 for an axisymmetric
        straight section.

    nev
        Event number to be associated with these stresses (defaults to 1).

    nlod
        Loading number to be associated with these stresses (defaults to
        1).

    kbr
        For an axisymmetric analysis (RHO ≠ 0):

        0 - Include the thickness-direction bending stresses

        1 - Ignore the thickness-direction bending stresses

        2 - Include the thickness-direction bending stress using the same formula as the Y
            (axial direction ) bending stress. Also use the same formula
            for the shear stress.

    Notes
    -----
    Calculates and stores the total linearized stress components at the
    ends of a section path [PATH] (as defined by the first two nodes with
    the PPATH command). The path must be entirely within the selected
    elements (that is, there must not be any element gaps along the path).
    Stresses are stored according to the fatigue event number and loading
    number specified.  Locations (one for each node) are associated with
    those previously defined for these nodes [FL] or else they are
    automatically defined.  Stresses are separated into six total
    components (SX through SXZ) and six membrane-plus-bending (SX through
    SXZ) components.  The temperature at each end point and the current
    time are also stored along with the total stress components.
    Calculations are made from the stresses currently in the database (last
    SET or LCASE command).  Stresses are stored as section coordinate
    components if axisymmetric or as global Cartesian coordinate components
    otherwise, regardless of the active results coordinate system [RSYS].
    The FSLIST command may be used to list stresses.  The FS command can be
    used to modify stored stresses.  See also the PRSECT and PLSECT
    commands for similar calculations.
    """
    command = f"FSSECT,{rho},{nev},{nlod},{kbr}"
    return self.run(command, **kwargs)


def fscale(self, rfact="", ifact="", **kwargs):
    """Scales force load values in the database.

    APDL Command: FSCALE

    Parameters
    ----------
    rfact
        Scale factor for the real component.  Zero (or blank) defaults to
        1.0.  Use a small number for a zero scale factor.

    ifact
        Scale factor for the imaginary component.  Zero (or blank) defaults
        to 1.0.  Use a small number for a zero scale factor.

    Notes
    -----
    Scales force load (force, heat flow, etc.) values in the database.
    Scaling applies to the previously defined values for the selected nodes
    [NSEL] and the selected force labels [DOFSEL].  Issue FLIST command to
    review results.  Solid model boundary conditions are not scaled by this
    command, but boundary conditions on the FE model are scaled.

    Note:: : Such scaled FE boundary conditions may still be overwritten by
    unscaled solid model boundary conditions if a subsequent boundary
    condition transfer occurs.

    FSCALE does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"FSCALE,{rfact},{ifact}"
    return self.run(command, **kwargs)



def rmndisp(self, loadt="", loc="", **kwargs):
    """Extracts neutral plane displacements from a test load or element load

    APDL Command: RMNDISP
    solution for the ROM method.

    Parameters
    ----------
    loadt
        Load type. Load type must be an alphanumeric string enclosed in
        single quotes. Valid load types are 'TLOAD' for the test load and
        'ELOAD' for the element load.

    loc
        Determines whether file will be overwritten or appended. Valid
        labels are 'WRITE' or 'APPEND'. Defaults to 'WRITE' for test load.

    Notes
    -----
    This command extracts the displacements at a neutral plane of a model.
    If  LoadT = 'TLOAD', extract displacements for a test load on a
    structure that represents the expected deflection state.  A test load
    is used to assist in the automatic mode selection for the ROM mode
    characterization. If LoadT = 'ELOAD', extract the neutral plane
    displacements for an element load that will be used in the use pass of
    a ROM analysis.  Typical element loads are gravity, and pressure
    loading. The element loads may be scaled [RMLVSCALE] during the use
    pass.

    The command requires a node component named "NEUN" to be defined. These
    nodes represent the nodes at the neutral plane of a structure (in the
    case of a stress-stiffened structure), or at any plane in the structure
    (non stress-stiffened case).

    For LoadT = 'TLOAD', node displacements are written to the file
    jobname.tld.  For LoadT = 'ELOAD', node displacements are written to
    the file jobname.eld. Up to 5 element load cases may be written to the
    file jobname.eld.

    This command is only valid in POST1.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"RMNDISP,{loadt},{loc}"
    return self.run(command, **kwargs)


def rmnevec(self, **kwargs):
    """Extracts neutral plane eigenvectors from a modal analysis for the ROM

    APDL Command: RMNEVEC
    method.

    Notes
    -----
    This command extracts the eigenvectors at a neutral plane of a model
    from a modal analysis.  The modal analysis must have expanded modes
    [MXPAND] in order to process the data. Only the first 9 modes are
    considered.  The command requires a node component named "NEUN" to be
    defined. These nodes represent the nodes at the neutral plane of a
    structure (in the case of a stress-stiffened structure), or at any
    plane in the structure (non stress-stiffened case).

    This command is only valid in POST1.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.

    jobname.evx, jobname.evy, jobname.evz, jobname.evn, jobname.evl
    """
    command = f"RMNEVEC,"
    return self.run(command, **kwargs)



def igesin(self, fname="", ext="", **kwargs):
    """Transfers IGES data from a file into ANSYS.

    APDL Command: IGESIN

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Reads a file containing IGES data and transfers it into the ANSYS
    database.  The file transferred is the IGES Version 5.1, ASCII format
    file.  IGES (Initial Graphics Exchange Specification) is a neutral
    format developed by the U.S. Dept. of Commerce, National Institute of
    Standards and Technology.  There is no output transfer file written
    since the transferred data is read directly into the ANSYS database.

    You can import multiple files into a single database, but you must use
    the same import option (set with the IOPTN command) for each file.

    The IOPTN command sets the parameters for reading the file.  Files read
    via the SMOOTH method (the only available method) use the standard
    database.
    """
    return self.run(f"IGESIN,{fname},{ext}", **kwargs)


def ioptn(self, lab="", val1="", **kwargs):
    """Controls options relating to importing a model.

    APDL Command: IOPTN

    Parameters
    ----------
    lab
        Label identifying the import option.  The meaning of VAL1 varies
        depending on Lab.

        STAT - List overall status of import facilities, including current option values.
               VAL1 is ignored.

        DEFA - Set default values for all import options.  VAL1is ignored.

        MERG - Entity merge option.  VAL1 can be:

        YES - Automatic merging is performed (default).

        NO - No merging of entities.

        SOLID - Solid option.  VAL1 can be:

        YES - Solid is created automatically (default).

        NO - No solid created.

        GTOLER - Entity merging tolerance. If IGES = SMOOTH, the GTOLER,VAL1 can be:

        DEFA - Use system defaults (default).

        FILE - Use tolerance from the imported file.

        n - A user-specified tolerance value.

        IGES - IGES import option.  VAL1 can be:

        STAT - List status of IGES related options in the output window.

        SMOOTH (or RV52) - Use more robust IGES revision 5.2 import function (default).

        SMALL - Small areas option.   VAL1 can be:

        YES - Small areas are deleted (default).

        NO - Small areas are retained.

    val1
        Additional input value as described under each Lab option.

    Notes
    -----
    Controls various options during a model file transfer.  A global solid
    model tolerance (GTOLER) can be specified.

    The SMALL,YES option (default) delete small areas  and can cause
    geometrical inconsistencies that could cause the import process to
    abort.  Retaining the small areas increases processor time and memory
    usage.

    The data is stored in the standard ANSYS graphics database.

    The IGES,SMOOTH (default) option is capable of reading in any rational
    B-spline curve entity (type 126), or rational B-spline surface entity
    (type 128) with a degree less than or equal to 20.  Attempts to read in
    B-spline curve or surface entities of degree higher than 20 may result
    in error messages.

    If you issue the /CLEAR command, the IOPTN settings return to their
    defaults.

    For MERG,YES, merging of coincident geometry items is performed
    automatically when the IGESIN command is issued (that is, an internal
    NUMMRG,KP command is issued).  The model is merged with the
    consideration tolerance (TOLER on NUMMRG) set equal to 0.75 * the
    shortest distance between the endpoints of any active line. See the
    NUMMRG command for more information about the tolerances. In most
    cases, the default merging is appropriate.  Use the IOPTN command when
    you want to:

    Disable merging operations.

    Override the default merging and specify a global solid model tolerance
    value (GTOLER).

    Disable the automatic creation of solids (SOLID).

    The IOPTN command should be issued before the IGESIN command. You
    cannot change these options after your model has been imported or
    created. If you must change the options:

    Clear the database (/CLEAR) or exit and restart the program.

    Set the correct options.

    Reimport or recreate the model.

    This command is valid in any processor.
    """
    command = f"IOPTN,{lab},{val1}"
    return self.run(command, **kwargs)
